<?php

/**
 * @codeCoverageIgnore
 */
class TrafficHistRealtimeWorker extends TemplateWorker
{
    protected $sgbd;

    public $runtimeRegister = array(
        'traffichistrealtime' => 'doQuery'
    );

    public function __construct()
    {
        parent::__construct();
        $this->traffic = new Traffic();

        $this->sgbd = OPCache::get('traffic-sgbd', $this->sgbd);
        if ($this->sgbd === false) {
            $ret = do_query(
                "SHOW VARIABLES WHERE Variable_name='version' AND Value LIKE '%MariaDB%'",
                DBOPMON
            );
            $this->sgbd = ($ret->EOF) ? 'mysql' : 'mariadb';
            OPCache::set('traffic-sgbd', $this->sgbd, 86400);
        }

        $this->debug("Running over ".$this->sgbd);
    }

    public function doQuery($job)
    {
        $query = gzuncompress($job->workload());
        do_query($query, DBTRAFFIC);
        $this->removeRealtimeOldData();
    }

    protected function removeRealtimeOldData()
    {
        $dt = new DateTime();

        if ($this->sgbd == "mariadb") {
            $dt->modify("-10 minutes");
            $partition = sprintf("%d", $dt->format('i') / 5);
            $query = sprintf(
                "ALTER TABLE traffic.realtime ".
                "TRUNCATE PARTITION p%d",
                $partition
            );
        } else {
            $dt->modify("-5 minutes");
            $entry_time = $dt->format('YmdHis');

            $query = sprintf(
                "DELETE FROM traffic.realtime WHERE entry_time < '%s'",
                $entry_time
            );
        }

        do_query($query, DBTRAFFIC);
    }
}
