<?php

/**
 * @codeCoverageIgnore
 */
class TrafficRemoveOldDataWorker extends TemplateWorker
{
    public $runtimeRegister = array(
        'trafficremoveolddata' => 'removeOldData'
    );

    /**
     *     This method remove the old traffic data from hour, day and month
     * tables.
     */
    public function removeOldData($job)
    {
        $this->removeOldDataByPeriod("hour");
        $this->removeOldDataByPeriod("day");
        $this->removeOldDataByPeriod("month");
    }

    /**
     * The "removeOldDataByPeriod" remove the a traffic table using a
     * period to choose the correct table.
     * @param String $period name. (hour, day, month)
     * @throw RuntimeException if the period name is invalid.
     */
    protected function removeOldDataByPeriod($period)
    {
        $period = strtolower($period);

        switch ($period) {
            case "hour":
                $table      = "aggregated_by_hour";
                $modifier   = "-24 hour";
                $date_diff  = 86400; // one day in seconds
                break;
            case "day":
                $table      = "aggregated_by_day";
                $modifier   = "-31 day";
                $date_diff  = 2678400; // 31 days in seconds
                break;
            case "month":
                $table      = "aggregated_by_month";
                $modifier   = "-1 month";
                $date_diff  = 5356800; // Two months (62 days) in seconds
                break;
            default:
                throw new RuntimeException(
                    "Method '" . __FUNCTION__ .
                    "' received a invalid period: " . $period
                );
        }

        $query  = sprintf(
            "
			SELECT entry_time
			FROM traffic.%s
			ORDER BY entry_time ASC LIMIT 1",
            $table
        );
        $ret = do_query($query, DBTRAFFIC);

        if ($ret->RecordCount() <= 0) {
            $ret->Close();
            return;
        }

        $ret = $ret->fetchAll();
        $now = new Datetime();
        $last_entry = DateTime::createFromFormat("YmdHis", $ret[0]["entry_time"]);

        if (((int)$now->format('U') - (int)$last_entry->format('U')) < $date_diff) {
            return;
        }

        $now->modify($modifier);
        $query = sprintf(
            "DELETE FROM traffic.%s WHERE entry_time < '%d'",
            $table,
            (int)$now->format('YmdHis')
        );

        do_query($query, DBTRAFFIC);
    }
}
