<?php

include_once(SGL_CORE_DIR . '/AjaxProvider.php');

class TrafficAjaxProvider extends SGL_AjaxProvider
{
    private $traffic;
    private $licensed=false;

    // @codingStandardsIgnoreStart
    public function TrafficAjaxProvider()
    {
        // @codingStandardsIgnoreEnd
        $this->traffic = new Traffic();

        $lic = $this->traffic->checkLicense();
        $this->licensed = $lic['licensed'];

        if (($this->licensed != true) && ($this->licensed != false)) {
            $this->licensed = false;
        }

        SGL::logMessage(null, PEAR_LOG_DEBUG);
        parent::SGL_AjaxProvider();
        $this->responseFormat = SGL_RESPONSEFORMAT_JSON;

        global $mod;
        $mod = "TRAFFIC";

        if (! AuthKey::verify(SGL_Request::singleton()->get("k"))) {
            OpInterface::doLogin();
        }
    }

    public function ports()
    {
        if ($this->traffic->runOnLocalhost()) {
            return $this->remote_ports();
        }

        $cfg = $this->traffic->getClusterConf();
        $url = sprintf(
            "https://%s/opmon/seagull/www/index.php/traffic/action/remote_ports/?_dc=%s&page=%d&start=%d&limit=%d",
            $cfg['ip']['value'],
            SGL_Request::singleton()->get("_dc"),
            SGL_Request::singleton()->get("page"),
            SGL_Request::singleton()->get("start"),
            SGL_Request::singleton()->get("limit")
        );

        $ret = http_request(
            $url,
            array(
             'filter'    => SGL_Request::singleton()->get("filter"),
             'operation' => SGL_Request::singleton()->get("operation"),
             'jsonData'  => SGL_Request::singleton()->get("jsonData"),
             'k' => AuthKey::generate(__FUNCTION__),
            ),
            false,
            true
        );

        $obj = json_decode($ret);
        if (json_last_error() !== JSON_ERROR_NONE) {
            $ret = new StdClass();
            $ret->success = false;
            return $ret;
        }

        return $obj;
    }

    public function removePort()
    {
        $request = array();
        $port = json_decode(file_get_contents('php://input'), true);
        $key = $port['port'] . $port['service'] . $port['protocol'];
        $request['removed'][$key] = $port;

        $response = new StdClass;
        $response->success = $this->traffic->updatePorts($request);

        return $response;
    }

    // @codingStandardsIgnoreStart
    public function remote_ports()
    {
        // @codingStandardsIgnoreEnd
        $start = SGL_Request::singleton()->get("start");
        $limit = SGL_Request::singleton()->get("limit");
        $filter = SGL_Request::singleton()->get("filter");
        $operation = SGL_Request::singleton()->get("operation");
        $ret = new StdClass();
        $obj = json_decode(sgl_request::singleton()->get("jsonData"), true);

        if (json_last_error() !== JSON_ERROR_NONE) {
            $ret->success = false;
            return $ret;
        }

        if ($operation === 'save_changes') {
            $ret->success = $this->traffic->updatePorts($obj);
        } else {
            $ret->ports = $this->traffic->getPorts($filter);
            $ret->total = count($ret->ports);
            $ret->ports = array_slice($ret->ports, $start, $limit);
            $ret->success = true;
        }

        return $ret;
    }


    // @codingStandardsIgnoreStart
    public function check_license()
    {
        // @codingStandardsIgnoreEnd
        return $this->traffic->checkLicense();
    }


    // @codingStandardsIgnoreStart
    public function first_execution()
    {
        // @codingStandardsIgnoreEnd
        $obj = json_decode(SGL_Request::singleton()->get("jsonData"), true);

        if ($obj['end'] === true) {
            $this->traffic->setEndOfFirstExecution();
        }

        $ret = new StdClass();
        $ret->first_execution = $this->traffic->isFirstExecution();
        return $ret;
    }

    public function probes()
    {
        $request    =  SGL_Request::singleton();
        $data       = file_get_contents('php://input');
        $operation  = $request->get("operation");

        $failedResponse = new stdClass();
        $failedResponse->success = false;

        $data = json_decode(
            empty($data) ? $request->get("jsonData") : $data
        );

        if ($this->traffic->runOnLocalhost()) {
            return $this->remote_probes($operation, $data);
        }

        $cfg = $this->traffic->getClusterConf();
        $url = sprintf(
            "https://%s/opmon/seagull/www/index.php/traffic/action/remote_probes/?_dc=%s&node=root",
            $cfg['ip']['value'],
            SGL_Request::singleton()->get("_dc")
        );

        $response = http_request(
            $url,
            array(
                'operation' => $operation,
                'jsonData'  => json_encode($data),
                'k' => AuthKey::generate(__FUNCTION__),
            ),
            false,
            true
        );

        $obj = json_decode($response);
        if (! $obj) {
            return $failedResponse;
        }

        return $obj;
    }

    public function remote_probes($operation = null, $data = null)
    {
        $operation = ($operation) ?: SGL_Request::singleton()->get("operation");
        $data = ($data) ?: json_decode(SGL_Request::singleton()->get("jsonData"));

        $response = new StdClass();

        if ($operation === 'save_changes') {
            $response->success = $this->traffic->updateProbes($data);
            return $response;
        }

        $probes = $this->traffic->getProbes();

        if (empty($probes)) {
            $lastUpdate = new Datetime('now');

            $localhost_probe         = new StdClass();
            $localhost_probe->ip     = "127.0.0.1";
            $localhost_probe->name   = "127.0.0.1-1";
            $localhost_probe->active = false;
            $localhost_probe->id     = 1;
            $localhost_probe->last_update = $lastUpdate->format('Y/m/d H:i:s');

            $probes[] = $localhost_probe;
        }

        $response->success  = true;
        $response->elements = $probes;
        $response->total    = count($probes);

        return $response;
    }


    // @codingStandardsIgnoreStart
    public function cluster_conf()
    {
        // @codingStandardsIgnoreEnd
        $ret = new StdClass();
        $obj = json_decode(SGL_Request::singleton()->get("jsonData"), true);
        $operation = SGL_Request::singleton()->get("operation");

        if (json_last_error() !== JSON_ERROR_NONE) {
            $ret->success = false;
            return $ret;
        }

        if ($operation !== "save_changes") {
            $ret->success = true;
            $ret->data = $this->traffic->getClusterConf();
            return $ret;
        }

        $orig_config = $this->traffic->getClusterConf(true);
        $ip = $orig_config['ip']['value'];
        if (isset($obj['ip']['overwrite']) &&
            $obj['ip']['overwrite'] &&
            isset($obj['ip']['value']) &&
            strlen($obj['ip']['value'])) {
            $ip = $obj['ip']['value'];
        }

        if (! $this->traffic->updateClusterIPConf($ip)) {
            $ret->success = false;
            $ret->data = $this->traffic->getClusterConf();
            return $ret;
        }


        $sflow_port = $orig_config['sflow_port']['value'];
        $port = $orig_config['port']['value'];

        $sflow_overwrite = ((isset($obj['sflow_port']['overwrite']))
            && ($obj['sflow_port']['overwrite']));

        $netflow_overwrite = ((isset($obj['port']['overwrite']))
            && ($obj['port']['overwrite']));


        if ($netflow_overwrite &&
            isset($obj['port']['value']) &&
            strlen($obj['port']['value'])) {
            $port = $obj['port']['value'];
        }

        if ($sflow_overwrite &&
            isset($obj['sflow_port']['value']) &&
            strlen($obj['sflow_port']['value'])) {
            $sflow_port = $obj['sflow_port']['value'];
        }

        if (! $this->traffic->updateClusterPortConf($port, 'pmacctd')) {
            $ret->success = false;
            $ret->proto = 'Netflow';
            $ret->message = "The selected port is already in use.";
            $ret->data = $this->traffic->getClusterConf();
            return $ret;
        }

        if (! $this->traffic->updateClusterPortConf($sflow_port, 'sfacctd')) {
            $ret->success = false;
            $ret->proto = 'SFlow';
            $ret->message = "The selected port is already in use.";
            $ret->data = $this->traffic->getClusterConf();
            return $ret;
        }

        $cfg = $this->traffic->getClusterConf();

        if ($this->traffic->runOnLocalhost()) {
            SGL_Request::singleton()->set('port', $port);
            SGL_Request::singleton()->set('sflow_port', $sflow_port);
            $obj = $this->remote_change_port();
        } else {
            $url = sprintf(
                "https://%s/opmon/seagull/www/index.php/traffic/action/remote_change_port/",
                $cfg['ip']['value']
            );
            $ret = http_request(
                $url,
                array(
                 'port'       => $port,
                 'sflow_port' => $sflow_port,
                 'k' => AuthKey::generate(__FUNCTION__),
                ),
                false,
                true
            );

            $obj = json_decode($ret);
            if (json_last_error() !== JSON_ERROR_NONE) {
                $ret = new StdClass();
                $ret->success = false;
                return $ret;
            }
        }

        if (! $obj->success) {
            /*ok, we've failed, let's fallback */
            $this->traffic->updateClusterPortConf($cfg['port']['value'], 'pmacctd');
            $this->traffic->updateClusterPortConf($cfg['sflow_port']['value'], 'sfacctd');
            $this->traffic->restart_traffic();
        }

        $obj->data = $this->traffic->getClusterConf();
        return $obj;
    }


    // @codingStandardsIgnoreStart
    public function remote_change_port()
    {
        // @codingStandardsIgnoreEnd
        $pmacctd = $this->traffic->updateClusterPortConf($port, 'pmacctd');
        $sfacctd = $this->traffic->updateClusterPortConf($sflow_port, 'sfacctd');

        return $this->traffic->restart_traffic();
    }


    // @codingStandardsIgnoreStart
    public function put_traffic_on_boot()
    {
        // @codingStandardsIgnoreEnd
        TaskRunner::run('traffic_on_boot', 'null');
        return json_decode('{"success": true}');
    }

    public function deleteProbe()
    {
        $request = json_decode(file_get_contents('php://input'));
        $probe = $request[0];
        $response = array('success' => false);

        if (! isset($probe->probe_id)) {
            $response['message'] = 'Probe id is missing';
            return $response;
        }

        try {
            $this->traffic->deleteProbe($probe->probe_id);
        } catch (Exception $e) {
            $response['message'] = $e->getMessage();
            return $response;
        }

        $response['success'] = true;
        return $response;
    }
}
