<?php

class TrafficMgr extends SGL_Manager {
	private $_traffic;
	private $licensed=FALSE;
	function TrafficMgr() {
		$this->_traffic = new Traffic();

		parent::SGL_Manager();

		// define the possible actions
		$this->_aActionsMapping = array (
			'ports' => array( 'ports' ),
			'filter' => array( 'filter' ),
			'first_execution' => array ('first_execution'),
			'check_license' => array ('check_license'),
			'get_probes' => array ('get_probes'),
			'get_ips' => array ('get_ips')
        );

    }

	function validate($req, &$input) {
		// Register module
		global $mod;
		$mod = "TRAFFIC";
		$input->action = $req->get('action');
		$input->req = $req;

		$lic = $this->_traffic->checkLicense();
		$this->licensed = $lic['licensed'];
		
		if (($this->licensed != TRUE) && ($this->licensed != FALSE)) {
			$this->licensed = FALSE;
        }

        if (! AuthKey::verify($req->get('k'))) {
            OpInterface::doLogin();
        }

		$this->validated = true;
	}

	function _cmd_check_license() {
		print json_encode($this->_traffic->checkLicense());
	}
	
	function _cmd_ports(&$input = false, &$output = false) {
		if (!$this->licensed) {
			print json_encode(Array("licensed" => FALSE));
			return;
		}
		
		print json_encode($this->_traffic->getPorts(json_decode($input->req->get('jsonData'))));
	}

	function _cmd_get_probes(&$input = false, &$output = false) {
		if (!$this->licensed) {
			print json_encode(Array("licensed" => FALSE));
			return;
		}
		
		print json_encode($this->_traffic->getProbes());
	}
	
	function _cmd_first_execution(){
		if (!$this->licensed) {
			print json_encode(Array("licensed" => FALSE));
			return;
		}
		
		if ($this->_traffic->isFirstExecution() == 1) {
			print '{"first_execution":true}';
		} else {
			print '{"first_execution":false}';
		}
	}
	
	function _cmd_filter(&$input = false, &$output = false) {
		if (!$this->licensed) {
			print json_encode(Array("licensed" => FALSE));
			return;
		}
		
		$params = json_decode($input->req->get('jsonData'), TRUE);
		print json_encode($this->_traffic->filter($params));
	}

	function _cmd_get_ips(&$input = false, &$output = false) {
		if (!$this->licensed) {
			print json_encode(Array("licensed" => FALSE));
			return;
		}
		
		$params = json_decode($input->req->get('jsonData'), TRUE);
		
		if ($params) {
			$params_senders = $params_receivers = $params;
		} else {
			$params_senders = $params_receivers = array('opmon_output' => true);
		}		

		$params_senders['group_by'] = 'ip_src';
		$params_receivers['group_by'] = 'ip_dst';

		$senders = $this->_traffic->filter($params_senders);
		$receivers = $this->_traffic->filter($params_receivers);

		$t = array_merge($senders,$receivers);
				
		$ips = array();
		$label = array();					
		foreach ($t as $value) {
			if (isset($value['sender'])) {
				$label[$value['sender']] = $value['sender_name'];
				$ips[] = $value['sender'];
			} else if (isset($value['receiver'])) {
				$label[$value['receiver']] = $value['receiver_name'];
				$ips[] = $value['receiver'];
			}
		}
		$t = array_keys(array_flip($ips));
		$ips = array();
		foreach($t as $value) {
			$ips[] = array('ip' => $value,'label'=>$label[$value]);
		}
		
		print json_encode($ips);
	}

}
